@@ -2,33 +2,30 @@
 """
 BLE Client for BlueZ on Linux
 """
-import inspect
-import logging
 import asyncio
+import logging
 import os
 import warnings
-from typing import Callable, Optional, Union
+from typing import Callable, Dict, Optional, Union, cast
 from uuid import UUID
 
 import async_timeout
+from dbus_fast.aio import MessageBus
+from dbus_fast.constants import BusType, ErrorType
+from dbus_fast.message import Message
+from dbus_fast.signature import Variant
 
-from dbus_next.aio import MessageBus
-from dbus_next.constants import BusType, ErrorType
-from dbus_next.message import Message
-from dbus_next.signature import Variant
-
-from bleak.backends.bluezdbus import defs
-from bleak.backends.bluezdbus.characteristic import BleakGATTCharacteristicBlueZDBus
-from bleak.backends.bluezdbus.manager import get_global_bluez_manager
-from bleak.backends.bluezdbus.scanner import BleakScannerBlueZDBus
-from bleak.backends.bluezdbus.utils import (
-    assert_reply,
-    extract_service_handle_from_path,
-)
-from bleak.backends.client import BaseBleakClient
-from bleak.backends.device import BLEDevice
-from bleak.backends.service import BleakGATTServiceCollection
-from bleak.exc import BleakDBusError, BleakError
+from ... import BleakScanner
+from ...exc import BleakDBusError, BleakError
+from ..characteristic import BleakGATTCharacteristic
+from ..client import BaseBleakClient, NotifyCallback
+from ..device import BLEDevice
+from ..service import BleakGATTServiceCollection
+from . import defs
+from .characteristic import BleakGATTCharacteristicBlueZDBus
+from .manager import get_global_bluez_manager
+from .scanner import BleakScannerBlueZDBus
+from .utils import assert_reply
 from .version import BlueZFeatures
 
 logger = logging.getLogger(__name__)
@@ -53,7 +50,7 @@ class BleakClientBlueZDBus(BaseBleakClient):
     def __init__(self, address_or_ble_device: Union[BLEDevice, str], **kwargs):
         super(BleakClientBlueZDBus, self).__init__(address_or_ble_device, **kwargs)
         # kwarg "device" is for backwards compatibility
-        self._adapter = kwargs.get("adapter", kwargs.get("device", "hci0"))
+        self._adapter: Optional[str] = kwargs.get("adapter", kwargs.get("device"))
 
         # Backend specific, D-Bus objects and data
         if isinstance(address_or_ble_device, BLEDevice):
@@ -73,13 +70,15 @@ class BleakClientBlueZDBus(BaseBleakClient):
         self._disconnecting_event: Optional[asyncio.Event] = None
         # used to ensure device gets disconnected if event loop crashes
         self._disconnect_monitor_event: Optional[asyncio.Event] = None
+        # map of characteristic D-Bus object path to notification callback
+        self._notification_callbacks: Dict[str, NotifyCallback] = {}
 
         # used to override mtu_size property
         self._mtu_size: Optional[int] = None
 
     # Connectivity methods
 
-    async def connect(self, **kwargs) -> bool:
+    async def connect(self, dangerous_use_bleak_cache: bool = False, **kwargs) -> bool:
         """Connect to the specified GATT server.
 
         Keyword Args:
@@ -93,7 +92,7 @@ class BleakClientBlueZDBus(BaseBleakClient):
             BleakDBusError: If there was a D-Bus error
             asyncio.TimeoutError: If the connection timed out
         """
-        logger.debug(f"Connecting to device @ {self.address} with {self._adapter}")
+        logger.debug("Connecting to device @ %s", self.address)
 
         if self.is_connected:
             raise BleakError("Client is already connected")
@@ -106,8 +105,11 @@ class BleakClientBlueZDBus(BaseBleakClient):
         # Find the desired device before trying to connect.
         timeout = kwargs.get("timeout", self._timeout)
         if self._device_path is None:
-            device = await BleakScannerBlueZDBus.find_device_by_address(
-                self.address, timeout=timeout, adapter=self._adapter
+            device = await BleakScanner.find_device_by_address(
+                self.address,
+                timeout=timeout,
+                adapter=self._adapter,
+                backend=BleakScannerBlueZDBus,
             )
 
             if device:
@@ -144,9 +146,10 @@ class BleakClientBlueZDBus(BaseBleakClient):
                     disconnecting_event.set()
 
         def on_value_changed(char_path: str, value: bytes) -> None:
-            if char_path in self._notification_callbacks:
-                handle = extract_service_handle_from_path(char_path)
-                self._notification_callbacks[char_path](handle, bytearray(value))
+            callback = self._notification_callbacks.get(char_path)
+
+            if callback:
+                callback(bytearray(value))
 
         watcher = manager.add_device_watcher(
             self._device_path, on_connected_changed, on_value_changed
@@ -155,24 +158,43 @@ class BleakClientBlueZDBus(BaseBleakClient):
 
         try:
             try:
-                async with async_timeout.timeout(timeout):
-                    reply = await self._bus.call(
-                        Message(
-                            destination=defs.BLUEZ_SERVICE,
-                            interface=defs.DEVICE_INTERFACE,
-                            path=self._device_path,
-                            member="Connect",
+                #
+                # The BlueZ backend does not disconnect devices when the
+                # application closes or crashes. This can cause problems
+                # when trying to reconnect to the same device. To work
+                # around this, we check if the device is already connected.
+                #
+                # For additional details see https://github.com/bluez/bluez/issues/89
+                #
+                if not manager.is_connected(self._device_path):
+                    async with async_timeout.timeout(timeout):
+                        reply = await self._bus.call(
+                            Message(
+                                destination=defs.BLUEZ_SERVICE,
+                                interface=defs.DEVICE_INTERFACE,
+                                path=self._device_path,
+                                member="Connect",
+                            )
                         )
-                    )
-                assert_reply(reply)
+                    assert_reply(reply)
 
                 self._is_connected = True
 
                 # Create a task that runs until the device is disconnected.
                 self._disconnect_monitor_event = asyncio.Event()
-                asyncio.ensure_future(self._disconnect_monitor())
+                asyncio.ensure_future(
+                    self._disconnect_monitor(
+                        self._bus, self._device_path, self._disconnect_monitor_event
+                    )
+                )
 
-                await self.get_services()
+                #
+                # We will try to use the cache if it exists and `dangerous_use_bleak_cache`
+                # is True.
+                #
+                await self.get_services(
+                    dangerous_use_bleak_cache=dangerous_use_bleak_cache
+                )
 
                 return True
             except BaseException:
@@ -211,7 +233,10 @@ class BleakClientBlueZDBus(BaseBleakClient):
             self._cleanup_all()
             raise
 
-    async def _disconnect_monitor(self) -> None:
+    @staticmethod
+    async def _disconnect_monitor(
+        bus: MessageBus, device_path: str, disconnect_monitor_event: asyncio.Event
+    ) -> None:
         # This task runs until the device is disconnected. If the task is
         # cancelled, it probably means that the event loop crashed so we
         # try to disconnected the device. Otherwise BlueZ will keep the device
@@ -222,16 +247,16 @@ class BleakClientBlueZDBus(BaseBleakClient):
         # task will still be running and asyncio complains if a loop with running
         # tasks is stopped.
         try:
-            await self._disconnect_monitor_event.wait()
+            await disconnect_monitor_event.wait()
         except asyncio.CancelledError:
             try:
                 # by using send() instead of call(), we ensure that the message
                 # gets sent, but we don't wait for a reply, which could take
                 # over one second while the device disconnects.
-                await self._bus.send(
+                await bus.send(
                     Message(
                         destination=defs.BLUEZ_SERVICE,
-                        path=self._device_path,
+                        path=device_path,
                         interface=defs.DEVICE_INTERFACE,
                         member="Disconnect",
                     )
@@ -257,9 +282,6 @@ class BleakClientBlueZDBus(BaseBleakClient):
         # Try to disconnect the System Bus.
         try:
             self._bus.disconnect()
-
-            # work around https://github.com/altdesktop/python-dbus-next/issues/112
-            self._bus._sock.close()
         except Exception as e:
             logger.error(
                 f"Attempt to disconnect system bus failed ({self._device_path}): {e}"
@@ -345,9 +367,7 @@ class BleakClientBlueZDBus(BaseBleakClient):
         )
         assert_reply(reply)
         if reply.body[0].value:
-            logger.debug(
-                f"BLE device @ {self.address} already paired with {self._adapter}"
-            )
+            logger.debug("BLE device @ %s is already paired", self.address)
             return True
 
         # Set device as trusted.
@@ -363,9 +383,7 @@ class BleakClientBlueZDBus(BaseBleakClient):
         )
         assert_reply(reply)
 
-        logger.debug(
-            "Pairing to BLE device @ {0} with {1}".format(self.address, self._adapter)
-        )
+        logger.debug("Pairing to BLE device @ %s", self.address)
 
         reply = await self._bus.call(
             Message(
@@ -471,9 +489,14 @@ class BleakClientBlueZDBus(BaseBleakClient):
 
     # GATT services methods
 
-    async def get_services(self, **kwargs) -> BleakGATTServiceCollection:
+    async def get_services(
+        self, dangerous_use_bleak_cache: bool = False, **kwargs
+    ) -> BleakGATTServiceCollection:
         """Get all services registered for this GATT server.
 
+        Args:
+            dangerous_use_bleak_cache (bool): Use cached services if available.
+
         Returns:
            A :py:class:`bleak.backends.service.BleakGATTServiceCollection` with this device's services tree.
 
@@ -486,7 +509,9 @@ class BleakClientBlueZDBus(BaseBleakClient):
 
         manager = await get_global_bluez_manager()
 
-        self.services = await manager.get_services(self._device_path)
+        self.services = await manager.get_services(
+            self._device_path, dangerous_use_bleak_cache
+        )
         self._services_resolved = True
 
         return self.services
@@ -758,65 +783,18 @@ class BleakClientBlueZDBus(BaseBleakClient):
 
     async def start_notify(
         self,
-        char_specifier: Union[BleakGATTCharacteristicBlueZDBus, int, str, UUID],
-        callback: Callable[[int, bytearray], None],
+        characteristic: BleakGATTCharacteristic,
+        callback: NotifyCallback,
         **kwargs,
     ) -> None:
-        """Activate notifications/indications on a characteristic.
-
-        Callbacks must accept two inputs. The first will be a integer handle of the characteristic generating the
-        data and the second will be a ``bytearray`` containing the data sent from the connected server.
-
-        .. code-block:: python
-
-            def callback(sender: int, data: bytearray):
-                print(f"{sender}: {data}")
-            client.start_notify(char_uuid, callback)
-
-        Args:
-            char_specifier (BleakGATTCharacteristicBlueZDBus, int, str or UUID): The characteristic to activate
-                notifications/indications on a characteristic, specified by either integer handle,
-                UUID or directly by the BleakGATTCharacteristicBlueZDBus object representing it.
-            callback (function): The function to be called on notification.
         """
-        if not self.is_connected:
-            raise BleakError("Not connected")
+        Activate notifications/indications on a characteristic.
+        """
+        characteristic = cast(BleakGATTCharacteristicBlueZDBus, characteristic)
 
-        if inspect.iscoroutinefunction(callback):
+        self._notification_callbacks[characteristic.path] = callback
 
-            def bleak_callback(s, d):
-                asyncio.ensure_future(callback(s, d))
-
-        else:
-            bleak_callback = callback
-
-        if not isinstance(char_specifier, BleakGATTCharacteristicBlueZDBus):
-            characteristic = self.services.get_characteristic(char_specifier)
-        else:
-            characteristic = char_specifier
-
-        if not characteristic:
-            # Special handling for BlueZ >= 5.48, where Battery Service (0000180f-0000-1000-8000-00805f9b34fb:)
-            # has been moved to interface org.bluez.Battery1 instead of as a regular service.
-            # The org.bluez.Battery1 on the other hand does not provide a notification method, so here we cannot
-            # provide this functionality...
-            # See https://kernel.googlesource.com/pub/scm/bluetooth/bluez/+/refs/tags/5.48/doc/battery-api.txt
-            if str(char_specifier) == "00002a19-0000-1000-8000-00805f9b34fb" and (
-                self._hides_battery_characteristic
-            ):
-                raise BleakError(
-                    "Notifications on Battery Level Char ({0}) is not "
-                    "possible in BlueZ >= 5.48. Use regular read instead.".format(
-                        char_specifier
-                    )
-                )
-            raise BleakError(
-                "Characteristic with UUID {0} could not be found!".format(
-                    char_specifier
-                )
-            )
-
-        self._notification_callbacks[characteristic.path] = bleak_callback
+        assert self._bus is not None
 
         reply = await self._bus.call(
             Message(
