@@ -2,26 +2,24 @@
 
 import asyncio
 import logging
-from typing import List, Optional
+import sys
 import warnings
+from typing import List, Optional
+
+if sys.version_info[:2] < (3, 8):
+    from typing_extensions import Literal
+else:
+    from typing import Literal
 
 import async_timeout
-from typing_extensions import Literal
-
-from bleak.backends.scanner import (
-    AdvertisementDataCallback,
-    BaseBleakScanner,
-    AdvertisementData,
-)
-from bleak.backends.device import BLEDevice
-from bleak.exc import BleakError
-
 from android.broadcast import BroadcastReceiver
-from android.permissions import request_permissions, Permission
+from android.permissions import Permission, request_permissions
 from jnius import cast, java_method
 
-from . import defs
-from . import utils
+from ...exc import BleakError
+from ..device import BLEDevice
+from ..scanner import AdvertisementData, AdvertisementDataCallback, BaseBleakScanner
+from . import defs, utils
 
 logger = logging.getLogger(__name__)
 
@@ -46,9 +44,9 @@ class BleakScannerP4Android(BaseBleakScanner):
 
     def __init__(
         self,
-        detection_callback: Optional[AdvertisementDataCallback] = None,
-        service_uuids: Optional[List[str]] = None,
-        scanning_mode: Literal["active", "passive"] = "active",
+        detection_callback: Optional[AdvertisementDataCallback],
+        service_uuids: Optional[List[str]],
+        scanning_mode: Literal["active", "passive"],
         **kwargs,
     ):
         super(BleakScannerP4Android, self).__init__(detection_callback, service_uuids)
@@ -271,6 +269,11 @@ class _PythonScanCallback(utils.AsyncJavaCallbacks):
             entry.getKey().toString(): bytes(entry.getValue())
             for entry in record.getServiceData().entrySet()
         }
+        tx_power = result.getTxPower()
+
+        # change "not present" value to None to match other backends
+        if tx_power == defs.ScanResult.TX_POWER_NOT_PRESENT:
+            tx_power = None
 
         advertisement = AdvertisementData(
             local_name=record.getDeviceName(),
@@ -278,6 +281,7 @@ class _PythonScanCallback(utils.AsyncJavaCallbacks):
             service_data=service_data,
             service_uuids=service_uuids,
             platform_data=(result,),
+            tx_power=tx_power,
         )
         device = BLEDevice(
             device.getAddress(),
